@@ -77,7 +77,7 @@ def compute_max_events(signal, peak_rise, delta):
     return new_pks.astype(int)
 
 
-def correct_anomalies(peaks, alpha=0.05, save_name=''):
+def correct_anomalies(peaks, alpha=0.05, save_name=""):
     """
     Outlier peak detection (Grubb's test) and removal.
 
@@ -104,30 +104,38 @@ def correct_anomalies(peaks, alpha=0.05, save_name=''):
     from outliers import smirnov_grubbs as grubbs
 
     peak_diffs = abs(np.diff(peaks))
-    max_indices = grubbs.max_test_indices(peak_diffs, alpha=0.05)
-    # insert new peak halfway between too long RR interval
-    too_slow = np.array(max_indices)
-    new_peaks = np.zeros_like(too_slow, dtype=float)
 
-    for index, i in enumerate(too_slow):
-        # compute new diff_peak
-        new_diff = (peaks[i+1] - peaks[i])/2
-        # new peak to insert into corrected peaks array
-        new_peak = peaks[i] + new_diff
-        new_peaks[index] = new_peak
+    max_indices = grubbs.max_test_indices(peak_diffs, alpha=alpha)
+    min_indices = grubbs.min_test_indices(peak_diffs, alpha=alpha)
+    grubb_idxs = max_indices + min_indices
 
-    if np.any(too_slow):
-        corrected_peaks = np.insert(peaks,
-                                    (too_slow+1).reshape(-1),
-                                    new_peaks.reshape(-1))
-    else:
-        # No slow peaks detected, so returning the original array
-        corrected_peaks = peaks.copy()
+    # Compute representative difference based on its distribution
+    mean_rr = np.mean(
+        peak_diffs[[ii for ii in range(len(peak_diffs))
+                    if ii not in grubb_idxs]]
+        )
+    mean_rr = int(np.round(mean_rr))
 
+    corrected_peaks = peaks.copy()
+
+    for ix in max_indices:
+        n = int(np.round((peaks[ix + 1] - peaks[ix]) / mean_rr))
+
+        if n == 1:
+            continue
+
+        new_peaks = np.linspace(peaks[ix],
+                                peaks[ix + 1],
+                                n,
+                                dtype=int,
+                                endpoint=False)[1:]
+
+        corrected_peaks = np.append(corrected_peaks, new_peaks)
+
+    corrected_peaks = np.sort(corrected_peaks)
     corrected_peak_diffs = abs(np.diff(corrected_peaks))
-    mean_RR = np.mean(corrected_peak_diffs)
 
-    min_indices = grubbs.min_test_indices(corrected_peak_diffs, alpha=0.05)
+    min_indices = grubbs.min_test_indices(corrected_peak_diffs, alpha=alpha)
 
     # deleting peak such that resultant RR interval is furthest from mean RR
     # (i.e. gives longer RR interval)
@@ -143,34 +151,34 @@ def correct_anomalies(peaks, alpha=0.05, save_name=''):
             # if last RR interval (edge case)
             peaks_to_replace[index] = i  # replace first peak
             # compute new diff_peak
-            new_diff = (corrected_peaks[i+1] - corrected_peaks[i-1])/2
-            new_peaks2[index] = corrected_peaks[i-1] + new_diff
+            new_diff = (corrected_peaks[i + 1] - corrected_peaks[i - 1])/2
+            new_peaks2[index] = corrected_peaks[i - 1] + new_diff
         else:
             # replace first peak
-            new_diff1 = corrected_peaks[i+1] - corrected_peaks[i-1]
+            new_diff1 = corrected_peaks[i + 1] - corrected_peaks[i - 1]
             # replace second peak
-            new_diff2 = corrected_peaks[i+2] - corrected_peaks[i]
+            new_diff2 = corrected_peaks[i + 2] - corrected_peaks[i]
 
-            if (new_diff1 - mean_RR) > (new_diff2 - mean_RR):
+            if new_diff1 > new_diff2:
                 # replacing first peak results in new RR interval
                 # furthest from mean RR interval
                 peaks_to_replace[index] = i
                 # compute new diff_peak
-                new_diff = (corrected_peaks[i+1] - corrected_peaks[i-1])/2
-                new_peaks2[index] = corrected_peaks[i-1] + new_diff
+                new_diff = (corrected_peaks[i + 1] - corrected_peaks[i - 1])/2
+                new_peaks2[index] = corrected_peaks[i - 1] + new_diff
             else:
                 # replacing second peak results in new RR interval
                 # furthest from mean RR interval
-                peaks_to_replace[index] = i+1
+                peaks_to_replace[index] = i + 1
                 # compute new diff_peak
-                new_diff = (corrected_peaks[i+2] - corrected_peaks[i])/2
+                new_diff = (corrected_peaks[i + 2] - corrected_peaks[i])/2
                 new_peaks2[index] = corrected_peaks[i] + new_diff
 
     corrected_peaks2 = corrected_peaks.copy()
     np.put(corrected_peaks2, peaks_to_replace.astype(int), new_peaks2)
 
     # save peaks
-    if save_name != '':
-        np.savetxt(save_name, corrected_peaks2, delimiter=',')
+    if save_name != "":
+        np.savetxt(save_name, corrected_peaks2, delimiter=",")
 
     return corrected_peaks2, max_indices, min_indices
